# Generated by Django 2.2.10 on 2020-04-10 14:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_handler", "0009_forgottenpassword"),
    ]

    operations = [
        migrations.AddField(
            model_name="invitation",
            name="admin",
            field=models.BooleanField(default=False),
        ),
    ]
