# Generated by Django 2.2.5 on 2020-01-13 13:14

from django.db import migrations

import human_lambdas.workflow_handler.fields


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0009_auto_20200109_1621"),
    ]

    operations = [
        migrations.AlterField(
            model_name="task",
            name="outputs",
            field=human_lambdas.workflow_handler.fields.JSONField(default={}),
            preserve_default=False,
        ),
    ]
