# Generated by Django 2.2.5 on 2020-02-06 14:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_handler", "0005_invitation"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="name",
            field=models.CharField(max_length=128, null=True),
        ),
    ]
