# Generated by Django 2.2.5 on 2019-12-27 13:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_handler", "0003_auto_20191119_1829"),
    ]

    operations = [
        migrations.AlterField(
            model_name="organization",
            name="name",
            field=models.CharField(max_length=128, unique=True),
        ),
    ]
