# Generated by Django 2.2.10 on 2020-06-11 15:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0016_auto_20200504_2334"),
    ]

    operations = [
        migrations.AlterField(
            model_name="workflow",
            name="name",
            field=models.CharField(max_length=128),
        ),
    ]
