# Generated by Django 2.2.5 on 2019-11-19 14:57

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="task",
            name="completed",
            field=models.DateTimeField(blank=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="task",
            name="completed_by",
            field=models.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="user_handler.User",
            ),
        ),
    ]
