# Generated by Django 2.2.10 on 2020-09-18 14:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0026_taskactivity"),
    ]

    operations = [
        migrations.AlterField(
            model_name="taskactivity",
            name="source",
            field=models.CharField(
                choices=[
                    ("api", "api"),
                    ("zapier", "zapier"),
                    ("csv", "csv"),
                    ("manual", "manual"),
                ],
                max_length=64,
                null=True,
            ),
        ),
    ]
