# Generated by Django 2.2.10 on 2020-08-26 08:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0022_auto_20200824_1614"),
    ]

    operations = [
        migrations.AlterField(
            model_name="workflow",
            name="description",
            field=models.TextField(blank=True, null=True),
        ),
    ]
