# Generated by Django 2.2.10 on 2020-08-13 13:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0023_auto_20200813_1319"),
    ]

    operations = [
        migrations.RenameField(
            model_name="webhook",
            old_name="Workflow",
            new_name="workflow",
        ),
    ]
