# Generated by Django 2.2.13 on 2021-04-26 13:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0035_auto_20210319_1001"),
    ]

    operations = [
        migrations.AddField(
            model_name="workflow",
            name="pinned_block",
            field=models.TextField(null=True),
        ),
    ]
