# Generated by Django 2.2.10 on 2020-09-01 10:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0024_auto_20200826_0847"),
        ("workflow_handler", "0024_auto_20200813_1340"),
    ]

    operations = []
