# Generated by Django 2.2.10 on 2020-06-26 17:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("user_handler", "0012_auto_20200624_1449"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="notification",
            name="last_notified",
        ),
    ]
