# Generated by Django 2.2.5 on 2020-02-10 15:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0011_auto_20200129_1640"),
    ]

    operations = [
        migrations.AddField(
            model_name="workflow",
            name="n_tasks",
            field=models.IntegerField(default=0),
        ),
    ]
