# Generated by Django 2.2.5 on 2020-02-13 15:29

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_handler", "0005_invitation"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="user",
            name="is_admin",
        ),
        migrations.AddField(
            model_name="organization",
            name="admin",
            field=models.ManyToManyField(
                related_name="admin_user", to=settings.AUTH_USER_MODEL
            ),
        ),
        migrations.AddField(
            model_name="user",
            name="current_organization_id",
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name="organization",
            name="name",
            field=models.CharField(max_length=128),
        ),
    ]
