# Generated by Django 2.2.10 on 2020-08-13 13:19

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0022_webhook"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="workflow",
            name="hook_id",
        ),
        migrations.AddField(
            model_name="webhook",
            name="Workflow",
            field=models.ForeignKey(
                default=None,
                on_delete=django.db.models.deletion.CASCADE,
                to="workflow_handler.Workflow",
            ),
        ),
        migrations.AddField(
            model_name="webhook",
            name="is_zapier",
            field=models.BooleanField(default=False),
        ),
    ]
