# Generated by Django 2.2.10 on 2020-06-24 14:49

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_handler", "0011_auto_20200527_1448"),
    ]

    operations = [
        migrations.CreateModel(
            name="Notification",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("enabled", models.BooleanField(default=True)),
                ("last_notified", models.DateTimeField(null=True)),
            ],
        ),
        migrations.AddField(
            model_name="user",
            name="notifications",
            field=models.OneToOneField(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="user_handler.Notification",
            ),
        ),
    ]
