<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <meta http-equiv="X-UA-Compatible" content="ie=edge" />
    <link
      href="https://fonts.googleapis.com/css?family=Ubuntu&display=swap"
      rel="stylesheet"
    />
    <title>Human Lambdas Forgotten Password</title>
  </head>

  <body style="font-family: Ubuntu, sans-serif; padding-top: 80px;">
    <h1 style="width: 100%; text-align: center;">
      Reset your Human Lambdas password
    </h1>
    <h4 style="width: 100%; text-align: center;">If you did not mean to do this, you can safely ignore this email</h4>
    <div style="width: 100%;">
      <a style="margin: 0 auto; display: block; width: 184px; margin-left: calc(50% - 82px); background-color: #6648ee; border: none; height: 28px; font-size: 20px; border-radius: 12px; color: white; text-align: center; text-decoration: none;" href="{{ password_link }}" target="_blank">Reset password</a>
    </div>
  </body>
</html>
