# Generated by Django 2.2.5 on 2020-02-18 09:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("user_handler", "0007_user_is_superuser"),
        ("user_handler", "0006_auto_20200206_1457"),
    ]

    operations = []
