# Generated by Django 2.2.13 on 2020-11-03 17:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0029_auto_20201029_1449"),
    ]

    operations = [
        migrations.AddField(
            model_name="task",
            name="handling_time_seconds",
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name="task",
            name="session_started_at",
            field=models.DateTimeField(null=True),
        ),
    ]
