# Generated by Django 2.2.10 on 2020-06-24 14:49

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_handler", "0012_auto_20200624_1449"),
        ("workflow_handler", "0018_auto_20200618_0949"),
    ]

    operations = [
        migrations.CreateModel(
            name="WorkflowNotification",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("enabled", models.BooleanField(default=True)),
                (
                    "notification",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="user_handler.Notification",
                    ),
                ),
                (
                    "workflow",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="workflow_handler.Workflow",
                    ),
                ),
            ],
        ),
    ]
