# Generated by Django 2.2.13 on 2021-02-09 15:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0032_auto_20201118_1109"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="workflow",
            name="description",
        ),
    ]
