# Generated by Django 2.2.5 on 2019-11-19 15:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_handler", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="organization",
            name="user",
        ),
        migrations.AddField(
            model_name="organization",
            name="user",
            field=models.ManyToManyField(to="user_handler.User"),
        ),
    ]
