# Generated by Django 2.2.5 on 2019-12-30 20:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0006_auto_20191219_1150"),
    ]

    operations = [
        migrations.AlterField(
            model_name="workflow",
            name="description",
            field=models.TextField(blank=True),
        ),
    ]
