<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <meta http-equiv="X-UA-Compatible" content="ie=edge" />
    <link href="https://fonts.googleapis.com/css?family=Inter&display=swap" rel="stylesheet">
    <title>Human Lambdas Invite</title>
</head>
<body style="font-family: Inter UI, -apple-system, BlinkMacSystemFont, Roboto, Helvetica Neue , Helvetica, Arial, sans-serif; padding-top: 60px;">
    <div style="width: 100%; margin-bottom: 16px">
        <img style="margin: 0 auto; display: block; width: 128px; height: 128px; margin-left: calc(50% - 64px);" src="https://humanlambdasstatic.s3.eu-west-2.amazonaws.com/HL-logo.png" />
    </div>
    <h1 style="width: 100%; text-align: center;">
      You've been invited to join {{ organization_name }} by {{ invite_sender }}
    </h1>
    <div style="width: 100%; text-align: center; margin-bottom: 25px;">Once you accept, you will be able to work this organization's tasks</div>
    <div style="width: 100%;">
        <a style="margin: 0 auto; display: block; width: 164px; margin-left: calc(50% - 82px); background-color: #6648ee; border: none; height: 32px; line-height: 32px; font-size: 18px; border-radius: 4px; color: white; text-align: center; text-decoration: none;" href="{{ invite_link }}" target="_blank">Accept Invite</a>
    </div>
</body>
</html>
