# Generated by Django 2.2.5 on 2020-03-09 17:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("workflow_handler", "0014_auto_20200303_0949"),
    ]

    operations = [
        migrations.RenameField(
            model_name="task",
            old_name="completed_by",
            new_name="assigned_to",
        ),
        migrations.AddField(
            model_name="task",
            name="assigned_at",
            field=models.DateTimeField(null=True),
        ),
    ]
